/**
  (c) Copyright CERN 2005. All rights not expressly granted are reserved.
icecontrols.support@cern.ch

  SPDX-License-Identifier: LGPL-3.0-only
**/

#uses "unSystemIntegrity/unSystemIntegrity_alarmDescription.ctl"

/**@name LIBRARY: unSystemIntegrity_unManagerState.ctl

@author: Juan Manuel Pablo Rodriguez

Creation Date: 12/04/2022

version 1.0

Purpose:
	Track state of managers. If the manager is in Manual mode and running, an alarm is raised.

Usage: Public

PVSS manager usage: CTRL (WCCOActrl)

Constraints:
	. global variable: the following variables are global to the script
	. data point type needed: _UnSystemIntegrity
	. data point: unManagerState_systemIntegrity
*/

//--------------------------------------------------------------------------------------------------------------------------------
// constant declaration
//--------------------------------------------------------------------------------------------------------------------------------

const string unManagerState_pattern = "unManagerState";
const string monitorManualRunning_pattern = "monitorManualRunning";

// alert ranges of system integrity alarm datapoint
const int c_unSystemIntegrity_unManagerState_AlertLevel_OK                = 0;
const int c_unSystemIntegrity_unManagerState_AlertLevel_ManualRunning     = 10;

// indexes of settings in the timeout in .config.data
const int c_unSystemIntegrity_unManagerState_Timeout_Index                = 1;

// default values for .config.data
const int c_unSystemIntegrity_unManagerState_DefaultConfig_LoopDelay      = 60;
const dyn_string c_unSystemIntegrity_unManagerState_DefaultConfig_ManagerFilter = makeDynString(c_unSystemIntegrity_unManagerState_DefaultConfig_LoopDelay,
																						   "WCCILdata",
																						   "WCCILdist",
																						   "WCCILevent",
																						   "WCCILproxy",
																						   "WCCILredu",
																						   "WCCILsim",
																						   "WCCILsplit",
																						   "WCCOACMWClient",
																						   "WCCOACMWServer",
																						   "WCCOALaser",
																						   "WCCOAbacnet",
																						   "WCCOActrl",
																						   "WCCOAdim",
																						   "WCCOAdip",
																						   "WCCOAiec",
																						   "WCCOAiec61850",
																						   "WCCOAmod",
																						   "WCCOAmodsrv",
																						   "WCCOAmqtt",
																						   "WCCOAopcua",
																						   "WCCOAopcuasrv",
																						   "WCCOArdb",
																						   "WCCOAs7",
																						   "WCCOAs7plus",
																						   "WCCOAsnmp",
																						   "WCCOAsnmpa",
																						   "WCCOAvalarch");

// script variable declaration
dyn_string g_unSystemIntegrity_unManagerStateList;             // list of the unManagerState  dp checked

int g_unSystemIntegrity_unManagerState_monitorThreadId;          // ID of the monitoring thread
int g_unSystemIntegrity_unManagerState_loopDelay;              // Delay after each check cycle

string unManagerState_alarmDescription;
shared_ptr<unSystemIntegrity_UpdateAlarmDescription> unManagerState_updAlDesc = new unSystemIntegrity_UpdateAlarmDescription();

//@{

//------------------------------------------------------------------------------------------------------------------------
// unManagerState_systemIntegrityInfo
/** Return the list of systemAlarm pattern.

@par Constraints
  None

@par Usage
  Public

@par PVSS managers
  CTRL

@return list of systemAlarm pattern
*/
dyn_string unManagerState_systemIntegrityInfo()
{
  return makeDynString(monitorManualRunning_pattern);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_Initialize
/**
Purpose:
Get the list of unManagerState check by the systemIntegrity. The ones that are enabled.

@param dsResult: dyn_string, output, the list of enabled unManagerState that are checked

Usage: Public

PVSS manager usage: CTRL (WCCOActrl)

*/
unSystemIntegrity_unManagerState_Initialize(dyn_string &dsResult)
{
	dyn_string dsList;
	int len, i;
	string dpToCheck;
	bool enabled;

	// Get configuration data from unManagerState_systemIntegrityInfo
	dsList = dpNames(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern + "*", c_unSystemAlarm_dpType);
	len = dynlen(dsList);
	for(i=1; i<=len; i++) {
		dpToCheck = dpSubStr(dsList[i], DPSUB_DP);
		dpToCheck = substr(dpToCheck, strlen(c_unSystemAlarm_dpPattern),strlen(dpToCheck));
		dpGet(dsList[i]+".enabled", enabled);
		if(enabled)
			dynAppend(dsResult, dpToCheck);
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_HandleCommand
/**
Purpose:
handle any systemIntegrity command.

@param sDpe1: string, input, dpe
@param command: int, input, the systemIntegrity command
@param sDpe2: string, input, dpe
@param parameters: dyn_string, input, the list of parameters of the systemIntegrity command

Usage: Public

PVSS manager usage: CTRL (WCCOActrl)

*/
unSystemIntegrity_unManagerState_HandleCommand(string sDpe1, int command, string sDpe2, dyn_string parameters)
{
	dyn_string exceptionInfo;
	int i, len =dynlen(parameters);

	switch(command) {
		case UN_SYSTEMINTEGRITY_ADD: // add the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				unSystemIntegrity_unManagerState_checking(parameters[i], true, true, exceptionInfo);
			}
			if (dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern)) {
				unSystemIntegrity_unManagerState_addAlert();
			}
			unSystemIntegrity_unManagerState_manageMonitoringThread("start");
			break;
		case UN_SYSTEMINTEGRITY_DELETE: // delete the _UnSystemAlarm DP
			if (dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern))
				// First remove the progs path and disconnect the _Connections datapoint
				unSystemIntegrity_unManagerState_removeAlert();
			for(i=1; i<=len; i++) {
				unSystemIntegrity_unManagerState_checking(parameters[i], false, false, exceptionInfo);
				// remove dp from the alert list of applicationDP if it is in
				_unSystemIntegrity_modifyApplicationAlertList(c_unSystemAlarm_dpPattern+parameters[i], false, exceptionInfo);
				// delete the _UnSystemAlarm dps
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					dpDelete(c_unSystemAlarm_dpPattern+parameters[i]);
			}
			unSystemIntegrity_unManagerState_manageMonitoringThread("stop");
			break;
		case UN_SYSTEMINTEGRITY_ENABLE: // enable the _UnSystemAlarm DP
			for(i=1; i<=len; i++) {
				if (dpExists(c_unSystemAlarm_dpPattern+parameters[i]))
					unSystemIntegrity_unManagerState_checking(parameters[i], false, true, exceptionInfo);
			}
			if (dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern)) {
				unSystemIntegrity_unManagerState_addAlert();
			}
			unSystemIntegrity_unManagerState_manageMonitoringThread("start");
			break;
		case UN_SYSTEMINTEGRITY_DISABLE: // disable the _UnSystemAlarm DP
			if (dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern))
				// First remove the progs path and disconnect the _Connections datapoint
				unSystemIntegrity_unManagerState_removeAlert();
			for(i=1; i<=len; i++) {
				if(dpExists(c_unSystemAlarm_dpPattern+parameters[i])) {
					unSystemIntegrity_unManagerState_checking(parameters[i], false, false, exceptionInfo);
				}
			}
			delay(1);
			unSystemIntegrity_unManagerState_manageMonitoringThread("stop");
			break;
		case UN_SYSTEMINTEGRITY_DIAGNOSTIC: // give the list of _UnSystemAlarm DP and the state
			dpSetWait(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.commandResult", command,
					unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".diagnostic.result", g_unSystemIntegrity_unManagerStateList);
			break;
		default:
			break;
	}

	if(dynlen(exceptionInfo) > 0) {
		if(isFunctionDefined("unMessageText_sendException")) {
			unMessageText_sendException("*", "*", "unSystemIntegrity_unManagerState_HandleCommand", "user", "*", exceptionInfo);
		}
// handle any error uin case the send message failed
		if(dynlen(exceptionInfo) > 0) {
			DebugTN(g_SystemIntegrity_sSystemName+"unSystemIntegrity_unManagerState_HandleCommand", exceptionInfo);
		}
	}
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_checking
/**
Purpose:
This function register/de-register the callback funtion of unManagerState manager. This function can also create the _unSystemAlarm_unManagerState dp
with the alarm config but it cannot delete it.

	@param sDp: string, input, data point name
	@param bCreate: bool, input, true create the dp and the alarm config if it does not exist, enable it
	@param bEnable: bool, input, true do a dpConnect, false do a dpDisconnect
	@param exceptionInfo: dyn_string, output, exception are returned here

Usage: Internal

PVSS manager usage: CTRL (WCCOActrl)

*/

unSystemIntegrity_unManagerState_checking(string sDp, bool bCreate, bool bEnable, dyn_string &exceptionInfo)
{
  // Remove the system name
  string sDpToCheck = unSystemIntegrity_removeSystemName(sDp);
  string sSysAlarmDp = c_unSystemAlarm_dpPattern + sDpToCheck;

  if(bCreate)
  {
    if(!dpExists(sSysAlarmDp))
    {
      dpCreate(sSysAlarmDp, c_unSystemAlarm_dpType);
	}
	string sSysAlarmDpe = sSysAlarmDp + ".alarm";
    dpSetDescription(sSysAlarmDpe, "Monitor manager running in Manual Mode status");
    unSystemIntegrity_unManagerState_addAlertConfig(sSysAlarmDpe, exceptionInfo);
  }

  if(dynlen(exceptionInfo)<=0) {
    if (bEnable) {
      // register alarm description
      unSystemIntegrity_registerAlarmDescription(sSysAlarmDp + ".alarm", unManagerState_updAlDesc, exceptionInfo);
      classConnect("unSystemIntegrity_unManagerState_alarmDescriptionCB", unManagerState_updAlDesc, unSystemIntegrity_UpdateAlarmDescription::updateAlarmDescription);
    } else {
      // unregister alarm description
      classDisconnect("unSystemIntegrity_unManagerState_alarmDescriptionCB", unManagerState_updAlDesc, unSystemIntegrity_UpdateAlarmDescription::updateAlarmDescription);
    }

	   // add/remove integrity classes
    _unSystemIntegrity_setList(g_unSystemIntegrity_unManagerStateList, sSysAlarmDp, bEnable);

    // enable/disable and unmask/mask
    dpSetWait(sSysAlarmDp + ".enabled", bEnable);
    unAlarmConfig_mask(sSysAlarmDp + ".alarm", !bEnable, exceptionInfo);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_addAlertConfig
/**
* Add discrete alert config to system alarm datapoint element
*
* @param sSysAlarmDpe    input, dpe for which we want to create the alert config
*	@param exceptionInfo   output, exception are returned here
*/
private void unSystemIntegrity_unManagerState_addAlertConfig(string sSysAlarmDpe, dyn_string &exceptionInfo)
{
  // set discrete alarm config
  dyn_mixed alarmObject;
  fwAlertConfig_objectCreateDiscrete(
    alarmObject,
    makeDynString("OK",
                  "Bad"),
    makeDynString("*",
                  c_unSystemIntegrity_unManagerState_AlertLevel_ManualRunning),
    makeDynString("",
                  getSystemName()+"_unSystemIntegrityAlarm_lvl1."),
    "",
    makeDynString(""),
    "",
    false,
    makeDynBool(false,false),
    "",
    makeDynString("",""),
    exceptionInfo,
    false,
    false,
    "",
    false); //exception info returned here
  if(dynlen(exceptionInfo)){
	DebugTN(__FUNCTION__, __LINE__, exceptionInfo);
	return;
  }
  //set the alarm to the dpe
  fwAlertConfig_objectSet(sSysAlarmDpe, alarmObject, exceptionInfo);
  if(dynlen(exceptionInfo)){
	DebugTN(__FUNCTION__, __LINE__, exceptionInfo);
	return;
  }
  //activate it
  fwAlertConfig_activate(sSysAlarmDpe, exceptionInfo);
  if(dynlen(exceptionInfo)){
	DebugTN(__FUNCTION__, __LINE__, exceptionInfo);
	return;
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_manageMonitoringThread
/**
* Starts and Stops the thread in which the checking is done
*
* @param command         input, start or stop the thread
*/
private void unSystemIntegrity_unManagerState_manageMonitoringThread(string command)
{
  dyn_string dsList = dpNames(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern + "*", c_unSystemAlarm_dpType);

  switch(command)
  {
    case "start":
      if(g_unSystemIntegrity_unManagerState_monitorThreadId == 0)
      {
        if(dynlen(dsList)> 0)
        {
          g_unSystemIntegrity_unManagerState_monitorThreadId = startThread("unSystemIntegrity_unManagerState_checkAlert");
          //DebugN("***********STARTING THREAD*********");
        }
      }
      break;

    case "stop":
      for(int i =1; i<=dynlen(dsList); i++)
      {
        bool enable;
        dpGet(dsList[i] + ".enabled", enable);
        if(enable == true)
          return;
      }

      int err;
      if(g_unSystemIntegrity_unManagerState_monitorThreadId != 0)
      {
        err = stopThread(g_unSystemIntegrity_unManagerState_monitorThreadId);
        //DebugN("***********STOPPING THREAD*********");
      }

      if(err<0)
        DebugTN(__FUNCTION__, __LINE__, "unSystemIntegrity_unManagerState_manageMonitoringThread() - stopping thread failed!");
      else
        g_unSystemIntegrity_unManagerState_monitorThreadId = 0;

      break;

    default:
      break;
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_addAlert
/**
* Check if a manager in Manual mode is running from the list of managers
* by connecting to the /config/progs path file.
*/

private void unSystemIntegrity_unManagerState_addAlert()
{

  // Connect to the PROJ_PATH + "config/progs" path.
  int rc = fswAddPath(PROJ_PATH + "config/progs");
  if (rc != 0) {
	  DebugTN(__FUNCTION__, __LINE__, "The progs file path cannot be added or it has not been found", getLastError());
  }

  // Check if there is a change in the starting mode of the managers by monitoring the progs file
  rc = sysConnect("unSystemIntegrity_unManagerState_statusChanged", "fswPathChanged");
  if (rc != 0) {
	  DebugTN(__FUNCTION__, __LINE__, "The function to track the change in managers status cannot be linked", getLastError());
  }

  // Check if there is a change in the list of managers' types specified in the System Integrity configuration panel
  rc = dpConnect("unSystemIntegrity_unManagerState_statusChanged", FALSE, unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".config.data");
  if (rc != 0) {
	  DebugTN(__FUNCTION__, __LINE__, "The function to track changes in the unManager list of manager's types has failed", getLastError());
  }
}

private void unSystemIntegrity_unManagerState_removeAlert()
{
  int rc = fswRemovePath(PROJ_PATH + "config/progs");
  if (rc != 0) {
	  DebugTN(__FUNCTION__, __LINE__, "The progs file path cannot be removed or it has not been found", getLastError());
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_statusChanged
/**
* Callback function that is triggered if the starting mode of a manager has changed.
*/

void unSystemIntegrity_unManagerState_statusChanged(string eventName, string path)
{
  int iErrorCode;

  unSystemIntegrity_unManagerState_raiseAlarm(iErrorCode);
  unSystemIntegrity_unManagerState_writeErrorCode(iErrorCode);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_setDefaultManagerFilter
/**
* Set the hardcoded default manager types to .config.data
*/

void unSystemIntegrity_unManagerState_setDefaultManagerFilter() {
  dpSet(unManagerState_pattern + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", c_unSystemIntegrity_unManagerState_DefaultConfig_ManagerFilter);
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_managerState_checkAlert
/**
* Check in a periodic cycle if a manager with Manual mode is running
* from the list of managers in the console.
*/
private void unSystemIntegrity_unManagerState_checkAlert()
{
  while(dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern)) {
	  unSystemIntegrity_unManagerState_getTimeout();
	  delay(g_unSystemIntegrity_unManagerState_loopDelay);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_getTimeout
/**
* Obtain the configured timeout
*/

private void unSystemIntegrity_unManagerState_getTimeout()
{
  dyn_string configTimeout;
  int rc = dpGet(unManagerState_pattern + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data", configTimeout);
  if (rc != 0) {
	  DebugTN(__FUNCTION__, __LINE__, "Default time stored in " + unManagerState_pattern + UN_SYSTEMINTEGRITY_EXTENSION + ".config.data" + " cannot be obtained.", getLastError());
	  return;
  } else {
	  if (dynlen(configTimeout) == 0 || configTimeout[c_unSystemIntegrity_unManagerState_Timeout_Index] == "") {
	    g_unSystemIntegrity_unManagerState_loopDelay = c_unSystemIntegrity_unManagerState_DefaultConfig_LoopDelay;
	  } else {
	    g_unSystemIntegrity_unManagerState_loopDelay = (int)configTimeout[c_unSystemIntegrity_unManagerState_Timeout_Index];
	  }
  }

  if (dynlen(configTimeout) > 1) {
	  unSystemIntegrity_unManagerState_statusChanged("","");
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_raiseAlarm
/**
* Raise an alarm if a manager is running in manual mode.
*/

private void unSystemIntegrity_unManagerState_raiseAlarm(int &iErrorCode)
{
  dyn_string dsManagerFilter, dsManagerRunningInManual;
  string sManagerType, sManagerOption;
  int iManagerNumber, iManagerState, iManagerMode;

  // Initialize the alarm to OK
  unSystemIntegrity_unManagerState_presetErrorCode(iErrorCode, c_unSystemIntegrity_unManagerState_AlertLevel_OK, true);

  for (int i=0; i<pmonGetCount(); i++) {
	// Get the manager type, option, number, state, and mode
	sManagerType = pmonGetName(i);
	sManagerOption = pmonGetOptions(i);
	iManagerNumber = pmonGetNum(i);
	iManagerState = pmonGetState(i);
	iManagerMode = pmonGetStartMode(i);

	// Obtain the manager's filter list and check if the current manager is contained within it
	dpGet(unManagerState_pattern+UN_SYSTEMINTEGRITY_EXTENSION+".config.data", dsManagerFilter);
	int pos = dynContains(dsManagerFilter, sManagerType);

	// Check if Manager is in Manual mode and Running
	// Mode: Manual - 0, Once - 1, Always - 2
	// Status: Stopped - 0, Initializing - 1, Running - 2, Blocked - 3
	if (pos > 0 && iManagerMode == 0 && iManagerState == 2) {
	  string sManagerRunningInManual = "Manager: " + sManagerType + " num " + iManagerNumber;
	  dynAppend(dsManagerRunningInManual, sManagerRunningInManual);
	}
  }
  // Remove manager type duplicates in dsManagerRunningInManual
  dynUnique(dsManagerRunningInManual);

  if (!dsManagerRunningInManual.isEmpty())
	unSystemIntegrity_unManagerState_presetErrorCode(iErrorCode, c_unSystemIntegrity_unManagerState_AlertLevel_ManualRunning);

  // Sets the unManagerState alarm description
  unManagerState_alarmDescription = strjoin(dsManagerRunningInManual, " | ");
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_presetErrorCode
/**
* Compare error code and set it if is higher than current error code
*
* @param iErrorCode     input/output, current error code
* @param iNewCode       input, new error code
* @param bForceNewCode  output, false: Write iErrorCode only if iNewCode is higher, true: Always write iErrorCode
*/
private void unSystemIntegrity_unManagerState_presetErrorCode(int &iErrorCode, int iNewCode, bool bForceNewCode=false)
{
  if(bForceNewCode) {
    iErrorCode = iNewCode;
  }
  else {
    if(iNewCode > iErrorCode) {
      iErrorCode = iNewCode;
    }
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_writeErrorCode
/**
* Write error code to unSystemIntegrity unManagerState alarm datapoint
*
* @param iErrorCode     input, error code to write
*/
private void unSystemIntegrity_unManagerState_writeErrorCode(int iErrorCode)
{
  string dpe = c_unSystemAlarm_dpPattern + monitorManualRunning_pattern + ".alarm";
  int rc, prevErrorCode;
  if (dpExists(c_unSystemAlarm_dpPattern + monitorManualRunning_pattern)) {
    rc = dpGet(dpe, prevErrorCode);
    if (dynlen(getLastError()) == 0) {
      if (prevErrorCode != iErrorCode) {
        rc = dpSetWait(dpe, iErrorCode);
        if (dynlen(getLastError()) > 0) {
          DebugTN(__FUNCTION__, __LINE__, "Error code cannot be written to the alarm datapoint.", getLastError());
          return;
        }
      }
    } else {
	    DebugTN(__FUNCTION__, __LINE__, "Error code cannot be obtained from the " + dpe + " alarm datapoint.", getLastError());
	    return;
    }
  } else {
    return;
  }

  // If there is an alarm, and the alarm value is the same than the previous one, trigger an update of the alarm description
  // in case that different managers are running in manual mode now.
  if (iErrorCode!=0 && prevErrorCode == iErrorCode){
    dyn_dyn_anytype alertInfo;
    string query = "SELECT ALERT '_alert_hdl.._value' FROM '" + dpe + "'";
    rc = dpQuery(query, alertInfo);
    dyn_errClass err = getLastError();
    if (dynlen(err)) {
      DebugTN(__FUNCTION__, __LINE__, "Alert information cannot be extracted from the query", err);
      return;
    }

    unSystemIntegrity_AlarmDescriptionTimestamp myTimeStamp;
    int lenResults = dynlen(alertInfo);
    if (lenResults >= 1) {
      for (int i = 2; i <= lenResults; i++) {
        myTimeStamp.timeStamp = alertInfo[i][2];
        myTimeStamp.aTimeStamp = alertInfo[i][2];
        myTimeStamp.dpeAlert = getAIdentifier(alertInfo[i][2]);
        myTimeStamp.alarmValue = alertInfo[i][3];
        if (!dpExists(myTimeStamp.dpeAlert)) {
          DebugTN(__FUNCTION__, __LINE__, "Alert datapoint element " + myTimeStamp.dpeAlert + " does not exist or has been removed.");
	         return;
        }
      }
    }
    // Trigger alarm description update
    unManagerState_updAlDesc.triggerUpdateAlarmDesc(myTimeStamp.timeStamp, myTimeStamp.aTimeStamp, myTimeStamp.dpeAlert, myTimeStamp.alarmValue);
  }
}

//------------------------------------------------------------------------------------------------------------------------

// unSystemIntegrity_unManagerState_alarmDescriptionCB
/**
* Callback function to set the unManagerState alarm description in the alertSet when
* the updateAlarmDescription class event has been triggered
*
* @param timeStamp         input, time, alert timestamp
* @param aTimeStamp        input, atime, alert atimestamp
* @param dpeAlert          input, string, alert datapoint element
* @param alarmValue        input, int, alarm value number
*/

public void unSystemIntegrity_unManagerState_alarmDescriptionCB(time timeStamp, atime aTimeStamp, string dpeAlert, int alarmValue) {
  unSystemIntegrity_setAlarmDescription(unManagerState_alarmDescription, timeStamp, aTimeStamp, dpeAlert);
}

//@}
